/*******************************************************************************
 * This file is part of CSDS and was copied from SWIFT.
 * Copyright (c) 2021 loic.hausammann@epfl.ch
 *               SWIFT
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
#ifndef CSDS_CLOCKS_H
#define CSDS_CLOCKS_H

/* Config parameters. */
#include "../config.h"

/* System includes. */
#include <sys/times.h>

/* Local includes */
#include "csds_cycle.h"

/* Struct to record a time for the clocks functions. */
struct clocks_time {
#ifdef HAVE_CLOCK_GETTIME
  struct timespec time;
#else
  ticks time;
#endif
};

/* Ticks used as the start of time. */
extern ticks clocks_start_ticks;

void clocks_gettime(struct clocks_time *time);
double clocks_diff(struct clocks_time *start, struct clocks_time *end);
const char *clocks_getunit(void);

void clocks_set_cpufreq(unsigned long long freq);
unsigned long long clocks_get_cpufreq(void);
double clocks_from_ticks(ticks tics);
ticks clocks_to_ticks(double interval);
double clocks_diff_ticks(ticks tic, ticks toc);
const char *clocks_get_timesincestart(void);
double clocks_get_hours_since_start(void);

void clocks_get_cputimes_used(double *usertime, double *systime);
int clocks_random_seed(void);

#endif /* CSDS_CLOCKS_H */
