/*******************************************************************************
 * This file is part of CSDS.
 * Copyright (c) 2021 Loic Hausammann (loic.hausammann@epfl.ch)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

/* Include header */
#include "csds_cosmology.h"

/* Include CSDS */
#include "csds_parser.h"
#include "csds_tools.h"

/**
 * @brief Compute the first time derivative of the scale factor
 *
 * @param cosmo The cosmological model
 * @param scale The scale factor
 *
 * @return The first derivative of the scale factor.
 */
void csds_cosmology_init(struct csds_cosmology *cosmo,
                         struct csds_parser *params) {

  /* Read Omega_cdm */
  cosmo->Omega_cdm = parser_get_param_double(params, "Cosmology:Omega_cdm");

  /* Read Omega_lambda */
  cosmo->Omega_lambda =
      parser_get_param_double(params, "Cosmology:Omega_lambda");

  /* Read Omega_b */
  cosmo->Omega_b = parser_get_param_double(params, "Cosmology:Omega_b");

  /* Read Omega_r */
  cosmo->Omega_r = parser_get_param_double(params, "Cosmology:Omega_r");

  /* Read Omega_k */
  cosmo->Omega_k = parser_get_param_double(params, "Cosmology:Omega_k");

  /* Read Omega_nu_0 */
  double Omega_nu_0 =
      parser_get_opt_param_double(params, "Cosmology:Omega_nu_0", 0);

  if (Omega_nu_0 != 0)
    error_python("Neutrinos are not implemented in the cosmology.");

  /* Read w_0 */
  cosmo->w_0 = parser_get_param_double(params, "Cosmology:w_0");

  /* Read w_a */
  cosmo->w_a = parser_get_param_double(params, "Cosmology:w_a");

  /* Read the Hubble constant at z=0 */
  cosmo->H0 = parser_get_param_double(params, "Cosmology:Hubble0");
}
