/*******************************************************************************
 * This file is part of CSDS.
 * Copyright (c) 2019 Loic Hausammann (loic.hausammann@epfl.ch)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
/**
 * @file csds_logfile.h
 * @brief This file contains the high level function for the log.
 */
#ifndef CSDS_CSDS_LOGFILE_H
#define CSDS_CSDS_LOGFILE_H

#include "csds_header.h"
#include "csds_mapped_file.h"
#include "csds_time.h"

struct csds_reader;

/**
 * @brief This structure deals with the log file.
 *
 * This structure is initialized by the #csds_reader
 * and deals with the log file.
 * It maps it, reverse the offsets (if required) and unmap it.
 *
 * The structure is initialized with #csds_logfile_init_from_file and
 * freed with #csds_logfile_free.
 */
struct csds_logfile {

  /* Information contained in the file header. */
  struct header header;

  /* The reader that is using this log file. */
  struct csds_reader *reader;

  /* Information about the time records. */
  struct time_array times;

  /* The file. */
  struct mapped_file log;
};

void csds_logfile_init_from_file(struct csds_logfile *log, char *filename,
                                 struct csds_reader *reader, int only_header);
void csds_logfile_reverse_offset(struct csds_logfile *log, char *filename);
void csds_logfile_free(struct csds_logfile *log);

#endif  // CSDS_CSDS_LOGFILE_H
