/*******************************************************************************
 * This file is part of CSDS.
 * Copyright (c) 2019 Loic Hausammann (loic.hausammann@epfl.ch)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

/**
 * @file csds_parameters.h
 * @brief This file contains the functions that deals with the yaml file.
 */

#ifndef CSDS_CSDS_PARAMETERS_H
#define CSDS_CSDS_PARAMETERS_H

/* Local include */
#include "csds_cosmology.h"
#include "csds_part_type.h"

/* Standard headers */
#include <stdint.h>

struct csds_reader;

/**
 * @brief Structure containing the simulation's parameters.
 */
struct csds_parameters {

  /* Number of dimension */
  int dimension;

  /* Simulation volume */
  double box_size[3];

  /* Is the volume periodic? */
  int periodic;

  /* Are we doing a cosmological simulation? */
  int with_cosmology;

  /* Number of particles initially present. */
  long long approximate_number_particles[csds_type_count];

  /* The cosmological model */
  struct csds_cosmology cosmo;

  /* Over allocation for the cache. */
  float cache_over_alloc;

  /* Initial cache for in-existing particles */
  int64_t cache_init_size[csds_type_count];
};

void csds_parameters_init(struct csds_parameters *params,
                          const struct csds_reader *reader);

#endif  // CSDS_CSDS_PARAMETERS_H
