/*******************************************************************************
 * This file is part of CSDS.
 * Copyright (c) 2019 Loic Hausammann (loic.hausammann@epfl.ch)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
/**
 * @brief This file contains functions that help to navigate in the logs.
 */
#ifndef CSDS_TOOLS_H
#define CSDS_TOOLS_H

/* Include config */
#include "../config.h"

/* Include local headers */
#include "csds_error.h"
#include "csds_inline.h"
#include "csds_logfile_writer.h"

/* Define the size of all the fields. */
#define member_size(type, member) sizeof(((type *)0)->member)

struct header;
struct csds_reader;
struct mapped_file;

/**
 * @brief Structure dealing with reading / interpolating a field.
 */
struct csds_reader_field {
  /* Value of the field. */
  void *field;

  /* Value of its first derivative (NULL if not existing). */
  void *first_deriv;

  /* Value of its second derivative (NULL if not existing). */
  void *second_deriv;
};

int tools_get_next_record(const struct header *h, struct mapped_file *map,
                          size_t *offset, size_t file_size);
int _tools_get_next_record_backward(const struct header *h,
                                    struct mapped_file *map, size_t *offset,
                                    size_t file_size);
int _tools_get_next_record_forward(struct mapped_file *map, size_t *offset);
size_t tools_reverse_offset(const struct header *h, struct mapped_file *map,
                            size_t offset);
size_t tools_check_record_consistency(struct csds_reader *reader,
                                      size_t offset);

void tools_print_progress(float percentage, const ticks tic,
                          const char *message);

#endif  // CSDS_TOOLS_H
